@@ -1,13 +1,22 @@
+//! TexCreate Config Library <br>
+//! This library contains all code related to `config.toml` <br>
+//! Developer: Mustafif Khan <br>
+//! License: MIT & GPLv2
+
+/// Contains all error handling using `failure` crate
 pub mod error;
+/// Contains all code related to the `README.md` & `texcreate.toml`
 pub mod extra;
 
-use std::fmt::format;
-use std::io::Write;
+use crate::TexCreateError::Invalid;
 use async_std::fs::*;
 use async_std::io::prelude::*;
 pub use error::*;
 pub use extra::*;
+use rocket::form::FromForm;
 use serde::{Deserialize, Serialize};
+use std::fmt::format;
+use std::io::Write;
 use std::path::Path;
 use std::path::PathBuf;
 use tex_rs::Latex;
@@ -15,14 +24,25 @@ use texc_latex::templates::*;
 use texcreate_lib::config::{Config as LegacyConfig, Document, Project};
 use texcreate_lib::Errors;
 use toml::{from_str, to_string_pretty};
+use zip::read::ZipFile;
 use zip::write::{FileOptions, ZipWriter};
 use zip::CompressionMethod::Stored;
-use zip::read::ZipFile;
-use crate::TexCreateError::Invalid;
-use rocket::form::FromForm;
 
 type F = std::fs::File;
 
+
+/// The Config struct contains all necessary metadata information
+/// - Author: The author of the project
+/// - Title: The title of the document
+/// - Date: The date for the document
+/// - Project Name: The name for the project
+/// - Template: The prebuilt template to use
+/// - Paper-size: The paper-size for the document
+/// - Document Class: The document class of the document
+/// - Font size: The font size for the document
+/// - Packages: Additional packages to add
+/// - Language: __Not supported yet__
+/// - Only Files: Whether structure is project or files _(default: false)_
 #[derive(Debug, Clone, Deserialize, Serialize, FromForm)]
 pub struct Config {
     pub author: String,
@@ -39,6 +59,7 @@ pub struct Config {
 }
 
 impl Default for Config {
+    /// Default settings for Config
     fn default() -> Self {
         Self {
             author: "Author".to_string(),
@@ -57,6 +78,7 @@ impl Default for Config {
 }
 
 impl Config {
+    /// Creates a new Config
     pub fn new(
         author: &str,
         title: &str,
@@ -84,12 +106,15 @@ impl Config {
             only_files,
         }
     }
+    /// Using `toml::to_string_pretty()`, turns Config into toml string for `config.toml`
     pub fn to_string(&self) -> Result<String, toml::ser::Error> {
         Ok(to_string_pretty(self)?)
     }
+    /// From `config.toml` string, turns into Config
     pub fn from_string(s: String) -> Result<Self, toml::de::Error> {
         Ok(from_str(&s)?)
     }
+    /// Turns a v1 `config.toml` into v2 Config toml string
     pub fn migrate() -> Result<String, toml::ser::Error> {
         let legacy = LegacyConfig::config(&None);
         let config = Config::new(
@@ -112,6 +137,8 @@ impl Config {
         );
         config.to_string()
     }
+
+    /// Grabs the `tex_rs::Latex` template from `&self.template` by matching the string
     pub fn template(&self) -> TexCreateResult<Latex> {
         let f = match &*self.template {
             "Basic" => basic(
@@ -121,6 +148,7 @@ impl Config {
                 &self.author,
                 &self.title,
                 &self.date,
+                &self.packages
             ),
             "Code" => code(
                 self.clone().font_size,
@@ -129,6 +157,7 @@ impl Config {
                 &self.author,
                 &self.title,
                 &self.date,
+                &self.packages
             ),
             "Novel" => novel(
                 self.clone().font_size,
@@ -137,15 +166,39 @@ impl Config {
                 &self.author,
                 &self.title,
                 &self.date,
+                &self.packages
+            ),
+            "Theatre" => theatre(
+                self.clone().font_size,
+                &self.paper_size,
+                &self.document_class,
+                &self.author,
+                &self.title,
+                &self.date,
+                &self.packages
+            ),
+            "Lachaise" => lachaise(
+                self.clone().font_size,
+                &self.paper_size,
+                &self.document_class,
+                &self.author,
+                &self.title,
+                &self.date,
+                &self.packages
             ),
             _ => return Err(TexCreateError::InvalidTemplate(self.template.clone())),
         };
         Ok(f)
     }
+    /// Builds project depending on `Config.only_files`
     pub async fn build(&self) -> TexCreateResult<()> {
         println!("Checking for any errors...");
         check_errors(&self)?;
         println!("Loading template: {}", &self.template);
+        if &self.template == "Book"{
+            book(&self.project_name, &self.title, &self.author).await.unwrap();
+            return Ok(());
+        }
         let latex = self.template()?;
         println!("Creating project: {}", &self.project_name);
         let path = Path::new(&self.project_name);
@@ -184,22 +237,33 @@ impl Config {
         }
         Ok(())
     }
-    pub async fn zip_proj(&self, path: PathBuf) -> TexCreateResult<()>{
-        let mut zip = ZipWriter::new(F::create(format!("{}/{}.zip",path.to_str().unwrap() ,&self.project_name)).unwrap());
+    /// Zips a project structure
+    pub async fn zip_proj(&self, path: PathBuf) -> TexCreateResult<()> {
+        let mut zip = ZipWriter::new(
+            F::create(format!(
+                "{}/{}.zip",
+                path.to_str().unwrap(),
+                &self.project_name
+            ))
+            .unwrap(),
+        );
         let options = FileOptions::default().compression_method(Stored);
 
         // README.md
         let readme_path = path.join("README.md");
         let _ = F::create(&readme_path).unwrap();
-        zip.start_file(readme_path.to_str().unwrap(), options).unwrap();
+        zip.start_file(readme_path.to_str().unwrap(), options)
+            .unwrap();
         zip.write_all(README.as_bytes()).unwrap();
         // texcreate.toml
         let toml_path = path.join("texcreate.toml");
         let _ = F::create(&toml_path).unwrap();
-        zip.start_file(toml_path.to_str().unwrap(), options).unwrap();
+        zip.start_file(toml_path.to_str().unwrap(), options)
+            .unwrap();
         let mut tex_toml = TexcToml::default();
         tex_toml.project_name = self.clone().project_name;
-        zip.write_all(to_string_pretty(&tex_toml).unwrap().as_bytes()).unwrap();
+        zip.write_all(to_string_pretty(&tex_toml).unwrap().as_bytes())
+            .unwrap();
         // out/
         let out_path = path.join("out");
         create_dir(&out_path).await?;
@@ -231,11 +295,18 @@ impl Config {
         remove_dir(&out_path).await?;
         remove_dir_all(&src).await?;
 
-
         Ok(())
     }
-    pub async fn zip_files(&self, path: PathBuf) -> TexCreateResult<()>{
-        let mut zip = ZipWriter::new(F::create(format!("{}/{}.zip",path.to_str().unwrap(), &self.project_name)).unwrap());
+    /// Zips a only files structure
+    pub async fn zip_files(&self, path: PathBuf) -> TexCreateResult<()> {
+        let mut zip = ZipWriter::new(
+            F::create(format!(
+                "{}/{}.zip",
+                path.to_str().unwrap(),
+                &self.project_name
+            ))
+            .unwrap(),
+        );
         let options = FileOptions::default().compression_method(Stored);
 
         let main = format!("{}/{}.tex", path.to_str().unwrap(), &self.project_name);
@@ -247,7 +318,8 @@ impl Config {
         zip.start_file(&main, options).unwrap();
         zip.write_all(&temp_str.0.as_bytes()).unwrap();
 
-        zip.start_file(structure.to_str().unwrap(), options).unwrap();
+        zip.start_file(structure.to_str().unwrap(), options)
+            .unwrap();
         zip.write_all(&temp_str.1.as_bytes()).unwrap();
 
         zip.finish().unwrap();
@@ -258,4 +330,4 @@ impl Config {
 
         Ok(())
     }
-}
\ No newline at end of file
+}
